/*
 * File Archive Library
 * Copyright (C) 2000-2001 Patrizio Bruno <patrizio@dada.it>
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    Read the file LICENSE for more.
 *
 */

/*
 * $Log: utils.c,v $
 * Revision 1.17  2001/05/28 12:08:42  rusty
 *
 * some ifdef/define/typdef corrected
 *
 * Revision 1.16  2001/05/28 11:44:33  rusty
 *
 * fixed a bug handling entry_info.st_* on solaris
 *
 * Revision 1.15  2001/05/28 11:12:19  rusty
 *
 * struct stat entry_info has been replaced by struct far_stat
 * a portable version of the stat structure
 *
 * Revision 1.14  2001/05/25 15:28:35  rusty
 *
 * fixed the struct dirent handling on solaris
 *
 * Revision 1.13  2001/05/25 15:03:55  rusty
 *
 * sun dirent support
 *
 * Revision 1.12  2001/05/24 16:59:33  rusty
 *
 * alphasort's wrapper
 *
 * Revision 1.11  2001/05/24 16:48:16  rusty
 *
 * scandir wrapper for systems that do not natively support this BSD function
 *
 * Revision 1.10  2001/05/21 12:16:46  rusty
 *
 * added a md5 checksum for the whole archive
 *
 * Revision 1.9  2001/05/21 11:06:31  rusty
 *
 * byte ordering fixed
 *
 * Revision 1.8  2001/05/16 16:08:17  rusty
 * *** empty log message ***
 *
 * Revision 1.7  2001/05/16 11:41:53  rusty
 *
 * code cleanup
 *
 * Revision 1.6  2001/05/16 11:34:03  rusty
 *
 * minor bug fixes
 *
 * Revision 1.5  2001/05/16 11:04:57  rusty
 *
 * some debug code removed
 * added a function displaying the internal version of the library: far_version() in utils.c
 *
 * Revision 1.4  2001/05/16 10:50:39  rusty
 *
 * fixed the checksum control on archive expand
 * some little code clean up
 *
 * Revision 1.3  2000/09/21 11:27:44  rusty
 *
 * bug fixes
 *
 * Revision 1.2  2000/09/20 17:43:07  rusty
 *
 * why checksums of big files seem to be corrupted?
 *
 * Revision 1.1  2000/09/20 14:10:22  rusty
 *
 * First working (basic) release
 *
 */

#include "far.h"
#include "utils.h"

extern char *arch_version;
extern char *unarch_version;

int is_adir(const char *path)
{
	struct	stat	check;

	memset(&check, 0, sizeof(check));
	lstat(path, &check);

	return S_ISDIR(check.st_mode);
}

int far_checksum(FAR_INFO *info, unsigned char *buffer)
{
	unsigned	char	sum[16];
	md5_state_t	md5;

	md5_init(&md5);
	md5_append(&md5, buffer, info->entry_info.fst_size);
	md5_finish(&md5, sum);

	return memcmp(info->chksum, sum, 16);
}

int far_file_cksum(unsigned char *buffer, size_t bufsize)
{
	unsigned	char	sum[16],
				*chksum	= (unsigned char *)NULL;
	md5_state_t	md5;

	md5_init(&md5);
	md5_append(&md5, buffer, (bufsize - 16));
	md5_finish(&md5, sum);

	chksum = (buffer + (bufsize - 16));

	return memcmp(chksum, sum, 16);
}

void make_list(char *string, char *delim, char *ilist[], int len)
{
	unsigned	int	finished	= 0, 
				dlen		= 0, 
				i		= 0;
	char			*ptr		= (char *)NULL;

	memset(ilist, 0, (sizeof(char*) * len + 5));
	ptr = string;
	dlen = strlen(delim);

	do
	{
		ilist[i] = ptr;

		if(((ptr = strstr(ptr, delim)) != NULL))
		{
			(*ptr) = 0;
			ptr += dlen;
			i++;
		}
		else
		{
			i++;
			ilist[i] = NULL;
			finished = 1;
		}
	} while(!finished && (i < len));
}

/* How many times does a substring appear in a string */
int strcount(char *haystack, char *needle)
{
	int	i	= 0;
	char	*ptr	= (char *)NULL;

	ptr = haystack;

	while((ptr = strstr(ptr, needle)) != NULL)
	{
		ptr += strlen(needle);
		i++;
	}

	return i;
}

char *strjoin(char **array, int n_memb, char *delim)
{
	size_t	tot_len	= 0;
	int	count	= 0;
	char	*result	= (char *)NULL;

	if(array && delim)
	{
		for(; count < n_memb ; count++)
		{
			tot_len += (strlen(array[count]) + strlen(delim));
		}

		tot_len++;
		result = calloc(1, tot_len);

		for(count = 0 ; count < n_memb ; count++)
		{
			strcat(result, array[count]);

			if(count != (n_memb - 1))
			{
				strcat(result, delim);
			}
		}
	}

	return result;
}

char *far_version(void)
{
	static	char	result[1024];

	sprintf(result, "%d.%d.%d%s%s",
		atoi(strpbrk(FAR_VERSION, "0123456789")),
		atoi(strchr(arch_version, '.') + 1),
		atoi(strchr(unarch_version, '.') + 1),
		(FARVER_BETA ? FARVER_BETA : ""),
		(FARVER_ALPHA ? FARVER_ALPHA : ""));

	return &result[0];
}

int far_printf(FILE *fp, char *fmt, ...)
{
	int	a	= 0;

	if(getenv("FAR_VERBOSE"))
	{
		va_list	ap;
		va_start(ap, fmt);
		a = vfprintf(fp, (const char *)fmt, ap);
		va_end(ap);
	}

	return a;
}

void far_byteorder_conv(FAR_INFO *info)
{
	info->entry_info.fst_mode = htonl(info->entry_info.fst_mode);
	info->entry_info.fst_uid = htonl(info->entry_info.fst_uid);
	info->entry_info.fst_gid = htonl(info->entry_info.fst_gid);
	info->entry_info.fst_rdev = far_conv64(info->entry_info.fst_rdev);
	info->entry_info.fst_atime = far_conv64(info->entry_info.fst_atime);
	info->entry_info.fst_mtime = far_conv64(info->entry_info.fst_mtime);
	info->entry_info.fst_size = far_conv64(info->entry_info.fst_size);
	info->offset = htonl(info->offset);
}

void far_byteorder_backconv(FAR_INFO *info)
{
	info->entry_info.fst_mode = ntohl(info->entry_info.fst_mode);
	info->entry_info.fst_uid = ntohl(info->entry_info.fst_uid);
	info->entry_info.fst_gid = ntohl(info->entry_info.fst_gid);
	info->entry_info.fst_rdev = far_conv64(info->entry_info.fst_rdev);
	info->entry_info.fst_atime = far_conv64(info->entry_info.fst_atime);
	info->entry_info.fst_mtime = far_conv64(info->entry_info.fst_mtime);
	info->entry_info.fst_size = far_conv64(info->entry_info.fst_size);
	info->offset = ntohl(info->offset);
}

int far_scandir(const char *dir, struct dirent ***namelist,
	int (*fselect)(const struct dirent *),
	int (*compar)(const struct dirent **, const struct dirent **))
{
#if	(!defined(_linux_) || !defined(_linux_))
	int		result	= -1;
	DIR		*dirp	= (DIR *)NULL;
	struct	dirent	**list	= (struct dirent **)NULL,
			*tmp	= (struct dirent *)NULL;

	if(namelist && dir)
	{
		dirp = opendir(dir);

		if(dirp)
		{
			 list = calloc(1, sizeof(struct dirent *));

			while((tmp = readdir(dirp)))
			{
				list = realloc(list, ((result + 2) *
						sizeof(struct dirent *)));

				if(fselect && fselect(tmp))
				{
					result++;
					list[result] = far_dirdup(tmp);
				}
				else if(!fselect)
				{
					result++;
					list[result] = far_dirdup(tmp);
				}
			}

			if(result > -1)
			{
				result++;
			}

			closedir(dirp);

			if(compar)
			{
				qsort(list, result, sizeof(struct dirent *),
					compar);
			}

			(*namelist) = list;
		}
	}

	return result;
#else
	return scandir(dir, namelist, select, compar);
#endif
}

int far_alphasort(const struct dirent **a, const struct dirent **b)
{
	int	result	= 0;

	if(a && b && *a && *b)
	{
		result = strcmp((*a)->d_name, (*b)->d_name);
	}

	return result;
}

void *memdup(void *src, size_t len)
{
	unsigned	char	*result	= (unsigned char *)NULL,
				*ptr	= (unsigned char *)src;

	if(ptr)
	{
		result = calloc(1, len);
		memcpy(result, ptr, len);
	}

	return result;
}

struct dirent *far_dirdup(struct dirent *dir)
{
	struct	dirent	*result	= (struct dirent *)NULL;

	if(dir)
	{
#if	(defined(_sun_) || defined(__sun__))
		result = calloc(1,
				(sizeof(struct dirent) + dir->d_reclen + 1));
		memcpy(result, dir,
			(sizeof(struct dirent) + dir->d_reclen + 1));
#else
		result = calloc(1, sizeof(struct dirent));
		memcpy(result, dir, sizeof(struct dirent));
#endif
	}

	return result;
}

u_int64_t far_conv64(u_int64_t a)
{
	char			tmp[8]		= { 0 };
	register	char	b	= 0;
	u_int64_t		result	= a;

#if	__BYTE_ORDER == __BIG_ENDIAN
	memcpy(tmp, &a, 8);

	b = tmp[0];
	tmp[0] = tmp[7];
	tmp[7] = b;

	b = tmp[1];
	tmp[1] = tmp[6];
	tmp[6] = b;

	b = tmp[2];
	tmp[2] = tmp[5];
	tmp[5] = b;

	b = tmp[3];
	tmp[3] = tmp[4];
	tmp[4] = b;

	memcpy(&result, tmp, 8);
#endif

	return result;
}

